
CUSTOM_SEARCH_TIME_PERIODS = [{
    "label": "Past day",
    "days": 1
}, {
    "label": "Past week",
    "days": 7
}, {
    "label": "Past 2 weeks",
    "days": 14
}, {
    "label": "Past month",
    "days": 31
}, {
    "label": "Past 6 months",
    "days": 31 * 6
}, {
    "label": "Past year",
    "days": 365
}]
